//
// file      : mdm_dev3.pas
// author    : Ian Forse
// date      : 18-APR-2004
// purpose   : Demo of ADC on MDM-1
// includes  : 
// notes     : 
// compiler  : P2C-plus 5.4e Pascal-Compiler
//
// Copyright (C) 2004 Ian Forse
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
// USA

program mdm_dev3;

asm __config 3F72h

// LOAD,DATA & CLOCK use Portb,2,4,5 respectively
const mdm_pin_load = 2;
const mdm_pin_data = 4;
const mdm_pin_clock = 5;

var mdm_cursor_position: char;
var channel_a: array[1..5] of char; 
var channel_b: array[1..5] of char; 
var segments_a, segments_b, a, b, c, p: char;

var adcon1@0x9F: char;
var adcon0@0x1F: char;
var pie1@0x8C: char;
var pir1@0x0C: char;
var adresl@0x9E: char;
var adresh@0x1E: char;
var ADC_DATA, ADC_VALUE, ADC_FILT: integer;
var ADC_CHANNEL: char;

// ----------------------------------------------------------------------------
// Channel A segment lookup table
// ----------------------------------------------------------------------------

const var tablea: array[0..127] of char := (
	00000000b, 	//  0x00   000   blank 
	11100010b,	//  0x01   001   o (degree)
	00011110b,	//  0x02   002   o (lower case o) 
	01000010b,	//  0x03   003   ohm 
	00011100b,	//  0x04   004   u 
	01001010b,	//  0x05   005   # (pound) 
	00010110b,	//  0x06   006   m 
	00000000b,	//  0x07   007   blank 
	00000000b,	//  0x08   008   blank 
	00000000b,	//  0x09   009   blank 
	00000000b,	//  0x0A   010   blank 
	00000000b,	//  0x0B   011   blank 
	00000000b,	//  0x0C   012   blank 
	00000000b,	//  0x0D   013   blank 
	00000000b,	//  0x0E   014   blank 
	00000000b,	//  0x0F   015   blank 
	00000000b,	//  0x10   016   blank 
	00000000b,	//  0x11   017   blank 
	00000000b,	//  0x12   018   blank 
	00000000b,	//  0x13   019   blank 
	00000000b,	//  0x14   020   blank 
	00000000b,	//  0x15   021   blank 
	00000000b,	//  0x16   022   blank 
	00000000b,	//  0x17   023   blank 
	00000000b,	//  0x18   024   blank 
	00000000b,	//  0x19   025   blank 
	00000000b,	//  0x1A   026   blank 
	00000000b,	//  0x1B   027   blank 
	00000000b,	//  0x1C   028   blank 
	00000000b,	//  0x1D   029   blank 
	00000000b,	//  0x1E   030   blank 
	00000000b,	//  0x1F   031   blank 
	00000000b,	//  0x20   032   space
	00001000b,	//  0x21   033   !
	00100000b,	//  0x22   034   "
	00110010b,	//  0x23   035   #
	01111010b,	//  0x24   036   $
	00110010b,	//  0x25   037   %
	00101110b,	//  0x26   038   &
	00000000b,	//  0x27   039   '
	00000000b,	//  0x28   040   (
	00000000b,	//  0x29   041   )
	00000010b,	//  0x2A   042   *
	00000010b,	//  0x2B   043   +        
	00000000b,	//  0x2C   044   , (clears dots)
	00000010b,	//  0x2D   045   -
	00000001b,	//  0x2E   046   .
	00000000b,	//  0x2F   047   /
	11111100b,	//  0x30   048   0     
	10010000b,	//  0x31   049   1
	11001110b,	//  0x32   050   2
	11011000b,	//  0x33   051   3
	10110010b,	//  0x34   052   4    
	01111010b,	//  0x35   053   5
	01111110b,	//  0x36   054   6
	11010000b,	//  0x37   055   7
	11111110b,	//  0x38   056   8
	11111010b,	//  0x39   057   9
	00000000b,	//  0x3A   058   :
	00000001b,	//  0x3B   059   ;
	00000000b,	//  0x3C   060   <
	00001010b,	//  0x3D   061   =
	00000000b,	//  0x3E   062   >
	11100001b,	//  0x3F   063   ?
	11101100b,	//  0x40   064   @
	11110110b,	//  0x41   065   A    
	11011000b,	//  0x42   066   B
	01101100b,	//  0x43   067   C
	11011000b,	//  0x44   068   D
	01101110b,	//  0x45   069   E
	01100110b,	//  0x46   070   F
	01111100b,	//  0x47   071   G
	10110110b,	//  0x48   072   H
	01001000b,	//  0x49   073   I
	10011100b,	//  0x4A   074   J
	00100110b,	//  0x4B   075   K
	00101100b,	//  0x4C   076   L
	10110100b,	//  0x4D   077   M
	10110100b,	//  0x4E   078   N
	11111100b,	//  0x4F   079   O
	11100110b,	//  0x50   080   P
	11111100b,	//  0x51   081   Q
	11100110b,	//  0x52   082   R
	01111010b,	//  0x53   083   S
	01000000b,	//  0x54   084   T
	10111100b,	//  0x55   085   U
	00100100b,	//  0x56   086   V
	10111100b,	//  0x57   087   W
	00000000b,	//  0x58   088   X
	00000000b,	//  0x59   089   Y
	01001000b,	//  0x5A   090   Z
	01101100b,	//  0x5B   091   [
	00000000b,	//  0x5C   092   \
	11011000b,	//  0x5D   093   ]
	00000000b,	//  0x5E   094   ^
	00001000b,	//  0x5F   095   _
	00000000b,	//  0x60   096   `
	11110110b,	//  0x61   097   a
	00111110b,	//  0x62   098   b
	00001110b,	//  0x63   099   c
	10011110b,	//  0x64   100   d
	11101110b,	//  0x65   101   e
	01100110b,	//  0x66   102   f
	11111010b,	//  0x67   103   g
	00100110b,	//  0x68   104   h
	00001010b,	//  0x69   105   i
	10011000b,	//  0x6A   106   j
	00100110b,	//  0x6B   107   k
	00100100b,	//  0x6C   108   l
	00010110b,	//  0x6D   109   m
	00000110b,	//  0x6E   110   n
	00011110b,	//  0x6F   111   o
	11100110b,	//  0x70   112   p
	11110010b,	//  0x71   113   q
	00000000b,	//  0x72   114   r
	00001000b,	//  0x73   115   s
	00101110b,	//  0x74   116   t
	00011100b,	//  0x75   117   u
	00000100b,	//  0x76   118   v
	00010100b,	//  0x77   119   w
	00000000b,	//  0x78   120   x
	10010000b,	//  0x79   121   y
	00001010b,	//  0x7A   122   z
	00000010b,	//  0x7B   123   {
	00000000b,	//  0x7C   124   |
	00000000b,	//  0x7D   125   }
	00110000b,	//  0x7E   126   ~
	11111111b);	//  0x7F   127   (del)

const var tableb: array[0..127] of char := (
// const char tableb[] = {
//        ihgnmlkr	     hex   dec   char
	00000000b,	//  0x00   000   blank 
	11100010b,	//  0x01   001   o (degree)
	00011110b,	//  0x02   002   o (lower case o) 
	01000010b,	//  0x03   003   ohm 
	00011100b,	//  0x04   004   u 
	01001010b,	//  0x05   005   # (pound) 
	00010110b,	//  0x06   006   m 
	00000000b,	//  0x07   007   blank 
	00000000b,	//  0x08   008   blank 
	00000000b,	//  0x09   009   blank 
	00000000b,	//  0x0A   010   blank 
	00000000b,	//  0x0B   011   blank 
	00000000b,	//  0x0C   012   blank 
	00000000b,	//  0x0D   013   blank 
	00000000b,	//  0x0E   014   blank 
	00000000b,	//  0x0F   015   blank 
	00000000b,	//  0x10   016   blank 
	00000000b,	//  0x11   017   blank 
	00000000b,	//  0x12   018   blank 
	00000000b,	//  0x13   019   blank 
	00000000b,	//  0x14   020   blank 
	00000000b,	//  0x15   021   blank 
	00000000b,	//  0x16   022   blank 
	00000000b,	//  0x17   023   blank 
	00000000b,	//  0x18   024   blank 
	00000000b,	//  0x19   025   blank 
	00000000b,	//  0x1A   026   blank 
	00000000b,	//  0x1B   027   blank 
	00000000b,	//  0x1C   028   blank 
	00000000b,	//  0x1D   029   blank 
	00000000b,	//  0x1E   030   blank 
	00000000b,	//  0x1F   031   blank   
	00000000b,	//  0x20   032   space
	01001000b,	//  0x21   033   !
	01000000b,	//  0x22   034   " 
	01001010b,	//  0x23   035   #
	01001010b,	//  0x24   036   $
	11001110b,	//  0x25   037   %
	01001000b,	//  0x26   038   &
	01000000b,	//  0x27   039   '
	10010000b,	//  0x28   040   (  
	00100100b,	//  0x29   041   ) 
	11111110b,	//  0x2A   042   * 
	01001010b,	//  0x2B   043   +
	00000000b,	//  0x2C   044   , clears dots)
	00000010b,	//  0x2D   045   -
	00000000b,	//  0x2E   046   .
	10000100b,	//  0x2F   047   /
	00000000b,	//  0x30   048   0  
	00000000b,	//  0x31   049   1  
	00000010b,	//  0x32   050   2  
	00000010b,	//  0x33   051   3  
	00000010b,	//  0x34   052   4  
	00000010b,	//  0x35   053   5  
	00000010b,	//  0x36   054   6  
	00000000b,	//  0x37   055   7  
	00000010b,	//  0x38   056   8  
	00000010b,	//  0x39   057   9  
	00000001b,	//  0x3A   058   :  
	00000001b,	//  0x3B   059   ;
	10010000b,	//  0x3C   060   <  
	00000010b,	//  0x3D   061   =  
	00100100b,	//  0x3E   062   >  
	00001010b,	//  0x3F   063   ?  
	01000010b,	//  0x40   064   @  
	00000010b,	//  0x41   065   A  
	01001010b,	//  0x42   066   B  
	00000000b,	//  0x43   067   C  
	01001000b,	//  0x44   068   D  
	00000000b,	//  0x45   069   E  
	00000000b,	//  0x46   070   F  
	00000010b,	//  0x47   071   G  
	00000010b,	//  0x48   072   H  
	01001000b,	//  0x49   073   I          
	00000000b,	//  0x4A   074   J 
	10010000b,	//  0x4B   075   K  
	00000000b,	//  0x4C   076   L  
	10100000b,	//  0x4D   077   M  
	00110000b,	//  0x4E   078   N  
	00000000b,	//  0x4F   079   O  
	00000010b,	//  0x50   080   P  
	00010000b,	//  0x51   081   Q  
	00010010b,	//  0x52   082   R  
	00000010b,	//  0x53   083   S  
	01001000b,	//  0x54   084   T  
	00000000b,	//  0x55   085   U  
	10000100b,	//  0x56   086   V  
	00001000b,	//  0x57   087   W  
	10110100b,	//  0x58   088   X  
	10101000b,	//  0x59   089   Y  
	10000100b,	//  0x5A   090   Z  
	00000000b,	//  0x5B   091   [  
	00110000b,	//  0x5C   092   \  
	00000000b,	//  0x5D   093   ] 
	10100000b,	//  0x5E   094   ^ 
	00000000b,	//  0x5F   095   _        
	00100000b,	//  0x60   096   `
	00000010b,	//  0x61   097   a
	00000010b,	//  0x62   098   b
	00000010b,	//  0x63   099   c
	00000010b,	//  0x64   100   d
	00000010b,	//  0x65   101   e
	00000000b,	//  0x66   102   f
	00000010b,	//  0x67   103   g
	00001000b,	//  0x68   104   h
	00001010b,	//  0x69   105   i
	00000000b,	//  0x6A   106   j
	00010010b,	//  0x6B   107   k
	00000000b,	//  0x6C   108   l
	00001010b,	//  0x6D   109   m
	00001000b,	//  0x6E   110   n
	00000010b,	//  0x6F   111   o
	00000010b,	//  0x70   112   p
	00000010b,	//  0x71   113   q
	00001010b,	//  0x72   114   r
	00010010b,	//  0x73   115   s
	00000000b,	//  0x74   116   t
	00000000b,	//  0x75   117   u
	00000100b,	//  0x76   118   v
	00010100b,	//  0x77   119   w
	10110100b,	//  0x78   120   x
	01000010b,	//  0x79   121   y
	00000100b,	//  0x7A   122   z
	10010000b,	//  0x7B   123   {
	01001000b,	//  0x7C   124   |
	00100110b,	//  0x7D   125   }
	00110000b,	//  0x7E   126   ~
	11111111b);	//  0x7F   127   (del)


// ----------------------------------------------------------------------------
// Shift byte x out to the display as serial.  n is number of bits (5 x 8) + 
// (1 * 3).  Note range is from 0 i.e. 8 is from 0 to 7.
// ----------------------------------------------------------------------------

procedure mdm_output_byte( var n, x: char);
var i: char;
begin
	for i := 0 to n do
	begin
		if x & 0x80 then
		begin
			set_bit(PORTB,mdm_pin_data);
		end;
		else
		begin
			clear_bit(PORTB,mdm_pin_data);
		end;
		set_bit(PORTB,mdm_pin_clock);
		clear_bit(PORTB,mdm_pin_clock);
		x := x <<1;
	end;
end;

// ----------------------------------------------------------------------------
// Update the display from the segment values.
// ----------------------------------------------------------------------------

procedure mdm_update_from_segments;
begin
	mdm_output_byte( 7, channel_a[5] );
	mdm_output_byte( 7, channel_a[4] );
	mdm_output_byte( 7, channel_a[3] );
	mdm_output_byte( 7, channel_a[2] );
	mdm_output_byte( 7, channel_a[1] );
	mdm_output_byte( 2, 0x00 );
	set_bit(PORTB,mdm_pin_load);
	clear_bit(PORTB,mdm_pin_load);
    
	mdm_output_byte( 7, channel_b[5] );
	mdm_output_byte( 7, channel_b[4] );
	mdm_output_byte( 7, channel_b[3] );
	mdm_output_byte( 7, channel_b[2] );
	mdm_output_byte( 7, channel_b[1] );
	mdm_output_byte( 2, 0x20 );
	set_bit(PORTB,mdm_pin_load);
	clear_bit(PORTB,mdm_pin_load);
end;


// ----------------------------------------------------------------------------
// Adjust segment values to either all on or all off.  Could be dropped if code
// space was tight.
// ----------------------------------------------------------------------------

procedure mdm_all( var data: char );
var i: char;
begin
	for i := 1 to 5 do
	begin
		channel_a[i] := data;
		channel_b[i] := data;
	end;
	mdm_update_from_segments;
end;


// ----------------------------------------------------------------------------
// write ascii character c to position p.
// ----------------------------------------------------------------------------

procedure mdm_write_p ( var p, c: char );
begin
	if p = 1 then
	begin
		segments_a := channel_a[1];
		segments_b := channel_b[1];
	end;
	if p = 2 then
	begin
		segments_a := channel_a[2];
		segments_b := channel_b[2];
	end;
	if p = 3 then
	begin
		segments_a := channel_a[3];
		segments_b := channel_b[3];
	end;  
	if p = 4 then
	begin
		segments_a := channel_a[4];
		segments_b := channel_b[4];
	end;   
	if p = 5 then
	begin
		segments_a := channel_a[5];
		segments_b := channel_b[5];
	end;

	if c = ',' then			// look for ',' == clear dots
	begin
		segments_a := segments_a & 11111110b;
		segments_b := segments_b & 11111110b;
	end;
	else
	begin
		c := c & 01111111b;	// tables are 128 entries only, so clear highest bit
		a :=  tablea[c];		// lookup channel a  
		b :=  tableb[c];		// lookup channel b
	end;
      
	if (( ( a | b ) & 00000001b ) = 0) then	// don't shift when writing the dots
	begin
		mdm_cursor_position := mdm_cursor_position + 1;
	end;

	segments_a := ( segments_a & 00000001b ) | a;	// don't overwrite the dots
	segments_b := ( segments_b & 00000001b ) | b;

	if p = 1 then
	begin
		channel_a[1] := segments_a;
		channel_b[1] := segments_b;
	end;
	if p = 2 then
	begin
		channel_a[2] := segments_a;
		channel_b[2] := segments_b;
	end;
	if p = 3 then
	begin
		channel_a[3] := segments_a;
		channel_b[3] := segments_b;
	end;
	if p = 4 then
	begin
		channel_a[4] := segments_a;
		channel_b[4] := segments_b;
	end;
	if p = 5 then
	begin
		channel_a[5] := segments_a;
		channel_b[5] := segments_b;
	end;
	mdm_update_from_segments;
end;

// ----------------------------------------------------------------------------
// clear the display, set cursor position to 1
// ----------------------------------------------------------------------------

procedure mdm_clear;
begin
	mdm_all( 0 );   
	mdm_cursor_position := 1;
end;


// ----------------------------------------------------------------------------
// write acii character c to cursor position, reset by mdm_clear.  Allows TTY
// style message to move scroll
// ----------------------------------------------------------------------------

procedure mdm_write ( var d: char );
begin
	if mdm_cursor_position > 4 then
	begin
		channel_a[1] := channel_a[2];
		channel_b[1] := channel_b[2];
		channel_a[2] := channel_a[3];
		channel_b[2] := channel_b[3];
		channel_a[3] := channel_a[4];
		channel_b[3] := channel_b[4];
		channel_a[4] := channel_a[5];
		channel_b[4] := channel_b[5];
		mdm_cursor_position := 5;
	end;
	mdm_write_p( mdm_cursor_position, d );
end;


// ----------------------------------------------------------------------------
// initialise the pin directions
// ----------------------------------------------------------------------------

procedure mdm_init;
begin
	output_low_port_b (mdm_pin_load);
	output_low_port_b (mdm_pin_data);
	output_low_port_b (mdm_pin_clock);
	set_tris_b (11001011b);
end;

// ----------------------------------------------------------------------------
// initialise the adc
// ----------------------------------------------------------------------------
procedure adc_init;
begin
	adcon1 := 10000000b;	// SELECT ONLY ONE FROM BELOW.
end;

// procedures for select combinations of ad inputs ( DS30292 page 122 )
// without external reference, right justified
         
// adcon1 = 10001110b;	// a0 adc input,a1,a2,a3,a5,e0,e1,e2 digital I/O
 
// adcon1 = 10000100b;	// a0,a1,a3 adc inputs,a2,a5,e0,e1,e2 digital I/O

// adcon1 = 10000010b;	// a0,a1,a2,a3,a4 adc inputs, a5,a6,a7 digital I/O

// adcon1 = 10001001b;	// a0,a1,a2,a3,a4,a5 adc inputs, a6,a7 digital I/O

// adcon1 = 10000000b;	// all adc inputs

 
// select combinations of ad inputs with positive reference on a3,
// right justified ( DS30292A page 121 )

// adcon1 = 10000101b;	// a0,a1 adc inputs,a4,a5,a6,a7 digital I/O

// adcon1 = 10000011b;	// a0,a1,a2,a4 adc inputs, a5,a6,a7 digital I/O

// adcon1 = 10001010b;	// a0,a1,a2,a4,a5 adc inputs, a6,a7 digital I/O

// adcon1 = 10000001b;	// all adc inputs


// select combinations of ad inputs with negative reference on a2 and positive 
// reference on a3, right justified ( DS30292A page 121 ) 


// adcon1 = 10001111b;	// a0 adc input, a1,a4,a5,a6,a7 digital I/O

// adcon1 = 10001101b;	// a0,a1 adc input, a4,a5,a6,a7 digital I/O
 
// adcon1 = 10001100b;	// a0,a1,a4 adc input, a5,a6,a7 digital I/O

// adcon1 = 10001010b;	// a0,a1,a4,a5 adc input, a6,a7 digital I/O
 
// adcon1 = 10001000b;	// a0,a1,a4,a5,a6,a7 adc input

// make all port_a digital IO

// adcon1 = 10000111b; // all digital IO


// ----------------------------------------------------------------------------
// Reads the ADC level input on a specified ADC channel 64 times.
// Approximate time for one sample = 581uS
// Averages value to remove noise
// Returns single 16bit word
// ----------------------------------------------------------------------------

function ReadADC( ADC_Channel: char ): integer;
var i: char;
begin
	if	ADC_Channel = 0 then adcon0 := 01000001b;
	if	ADC_Channel = 1 then adcon0 := 01001001b;
	if	ADC_Channel = 2 then adcon0 := 01010001b;
	if	ADC_Channel = 3 then adcon0 := 01011001b;
	if	ADC_Channel = 4 then adcon0 := 01100001b;
	if	ADC_Channel = 5 then adcon0 := 01101001b;
	if	ADC_Channel = 6 then adcon0 := 01110001b;
	if	ADC_Channel = 7 then adcon0 := 01111001b;

// channel 0 = Adj Pot on dev board
// channel 1 = temperature sensor on dev board
// channel 2 = 

		ADC_FILT := 0;			// Clear out previous count
		for i := 1 to 64 do
		begin
			clear_bit( pie1, 'ADIE' ); // Masking the interrupt
			clear_bit( pir1, 'ADIF' ); // Resetting the ADC interupt bit
			adresl	:= 0; 		// Resetting the ADRES value register
			adresh	:= 0;
			set_bit( adcon0, 'GO' );	  // Starting the ADC process
			while (!(pir1 & 0x40)) do;   // Wait for conversion complete
			ADC_VALUE := (adresh * 255) + adresl;	// convert from 2 bytes
			ADC_FILT := ADC_FILT + ADC_VALUE;		 	
		end;
		ADC_VALUE := ADC_FILT >>6;	// Average of 64 readings to reduce noise
		ADC_VALUE := (ADC_VALUE * 44) /9;	// Good approximation of convering
											// to 0 - 5V
		ReadADC := ADC_VALUE;		// Return the value of the ADC process
end;

begin	// Main
	adc_init();
	mdm_init();
	mdm_all(0);

	while 1 do
	begin
			ADC_CHANNEL := 0;
			ADC_DATA := ReadADC (ADC_CHANNEL);
			mdm_write_p (1, ( '0' + (ADC_DATA / 1000) % 10 ));
			mdm_write_p (2,'.');
			mdm_write_p (2,( '0' + (ADC_DATA / 100) % 10));
			mdm_write_p (3,( '0' + (ADC_DATA / 10) % 10 ));
			mdm_write_p (4,( '0' + ADC_DATA % 10 ));
			mdm_write_p (5,'V');
			delay_ms( 250 ); 
	end;
end.
